#include <sys/ioctl.h>
#include <fcntl.h>
#include <u/libu.h>
#include <fox/foxbone_t.h>
#include <fox/foxbone_userspace.h>

struct foxbone_s
{
    int fd; /* /dev/foxbone file descriptor */
};

int foxbone_addr_write(foxbone_t *fb, int addr, int data)
{
    FOXBONE_WRITE fwrite;

    dbg_err_if(fb == NULL);
    dbg_err_if(fb->fd < 0);

    /* zero-out (future) unknown struct fields */
    memset(&fwrite, 0, sizeof(FOXBONE_WRITE));

    fwrite.reg = addr;
    fwrite.data = data;

    dbg_err_sif(ioctl(fb->fd, IOCTL_FOXBONE_WRITE_REG, &fwrite) < 0);

    return 0;
err:
    return ~0;
}

int foxbone_addr_read(foxbone_t *fb, int addr, int *pdata)
{
    unsigned int data;

    dbg_err_if(fb == NULL);
    dbg_err_if(fb->fd < 0);

    dbg_err_sif((data = ioctl(fb->fd, IOCTL_FOXBONE_READ_REG, addr)) < 0);

    /* return the read word */
    *pdata = data;

    return 0;
err:
    return ~0;
}


int foxbone_version(foxbone_t *fb, int *prel, int *pw1, int *pw2, int *pw3)
{
    unsigned int rel, w1, w2, w3;

    dbg_err_if(fb == NULL);
    dbg_err_if(fb->fd < 0);

    /* foxbone release */
    dbg_err_sif((rel = ioctl(fb->fd, IOCTL_FOXBONE_READ_REG, 4)) < 0);

    /* first hardware application word */
    dbg_err_sif((w1 = ioctl(fb->fd, IOCTL_FOXBONE_READ_REG, 5)) < 0);

    /* second hardware application word */
    dbg_err_sif((w2 = ioctl(fb->fd, IOCTL_FOXBONE_READ_REG, 6)) < 0);

    /* third hardware application word */
    dbg_err_sif((w3 = ioctl(fb->fd, IOCTL_FOXBONE_READ_REG, 7)) < 0);

    /* success. set the output values */
    *prel = rel;
    *pw1 = w1;
    *pw2 = w2;
    *pw3 = w3;

    return 0;
err:
    return ~0;
}

int foxbone_reset(foxbone_t *fb)
{
    dbg_err_if(fb == NULL);
    dbg_err_if(fb->fd < 0);

    dbg_err_sif(ioctl(fb->fd, IOCTL_FOXBONE_INIT_BUS, 0) < 0);

    return 0;
err:
    return ~0;
}

int foxbone_open(foxbone_t **pfb)
{
    foxbone_t *fb = NULL;

    dbg_err_if(pfb == NULL);

    fb = u_zalloc(sizeof(foxbone_t));
    dbg_err_if(fb == NULL);

    /* open the device */
    while((fb->fd = open("/dev/foxbone", O_RDWR)) < 0 && errno == EINTR)
        /* nop */;

    /* log the strerror on error */
    dbg_err_sif(fb->fd < 0);

    /* return the obj */
    *pfb = fb;

    return 0;
err:
    if(fb)
        foxbone_close(fb);
    return ~0;
}

int foxbone_close(foxbone_t *fb)
{
    dbg_err_if(fb == NULL);

    if(fb->fd >= 0)
        close(fb->fd);

    u_free(fb);

    return 0;
err:
    return ~0;
}

